/*global define, CKEDITOR, setTimeout, document*/
/*jslint vars: true*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ckeditor/plugins/epalmostfullscreen/plugin-test',
    'jquery',
    'ep/ui/editor'

], function (QUnit, fixture, $, ep) {

    'use strict';
    QUnit.module("ckeditor/plugins/epallmostfullscreen", {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    function cke_test(instance, openselector, closeselector, setfunction, testfunction, after) {
        instance.setData('');
        $(openselector).trigger('click');
        setTimeout(function () {
            setfunction();
            setTimeout(function () {
                $(closeselector).trigger('click');
                setTimeout(function () {
                    var data = instance.getData();
                    testfunction(data);
                    after();
                }, 300);
            }, 200);
        }, 3000);
    }

    QUnit.asyncTest("default fullscreen test", function () {
        QUnit.fixtureVar.init();
        ep('#uieditor_ckeditortestinstance').uiEditor({
            'name' : 'ckeditortestinstance',
            'toolbar': 'Full'
        });
        setTimeout(function () {
            cke_test(CKEDITOR.instances.uieditor_ckeditortestinstance,
                '.cke_button__epalmostfullscreen_icon',
                '.cke_button__epalmostfullscreenclose_icon',
                function () {
                    QUnit.ok(
                        $('.ep-cke-full:visible').length === 1,
                        'Fullscreen mode is visible'
                    );
                    var buttons = $('#ep-cke-full-toolbar .cke_button .cke_button_label, #ep-cke-full-toolbar .cke_combo .cke_combo_label');

                    var labels = $.map(buttons, function (node) {
                        return node.innerHTML;
                    });

                    var expected_buttons = ["Rückgängig", "Wiederherstellen", "Suchen", "Format", "Schriftart", "Größe", "Formatierungen entfernen", "Fett", "Kursiv", "Unterstrichen", "Durchgestrichen", "Textfarbe", "Hintergrundfarbe", "Nummerierte Liste", "Liste", "Einzug verringern", "Einzug erhöhen", "Linksbündig", "Zentriert", "Rechtsbündig", "Blocksatz", "Zeilenabstand", "Bild", "App", "Link einfügen/editieren", "Link entfernen", "Tabelle", "Anker einfügen/editieren", "Horizontale Linie einfügen", "Sonderzeichen einfügen/editieren", "Blöcke anzeigen", "Quellcode", "Verkleinern"];

                    QUnit.deepEqual(
                        labels,
                        expected_buttons,
                        'Buttons in Fullscreen in the right order'
                    );
                },
                function (data) {
                    QUnit.ok(
                        $('.ep-cke-full:visible').length === 0,
                        'Fullscreen mode is not visible'
                    );
                },
                function () {
                    ep('#uieditor_ckeditortestinstance').uiEditor('destroy');
                    setTimeout(QUnit.start, 200);
                });
        }, 3000);
    });

    QUnit.asyncTest("blogpost fs test with pagebreak", function () {
        QUnit.fixtureVar.init();
        ep('#uieditor_ckeditortestinstance').uiEditor({
            'name' : 'ckeditortestinstance',
            'toolbar': 'FullPageBreak',
            'blogPostMode': true
        });
        setTimeout(function () {
            cke_test(CKEDITOR.instances.uieditor_ckeditortestinstance,
                '.cke_button__epalmostfullscreen_icon', '.cke_button__epalmostfullscreenclose_icon',
                function () {
                    QUnit.ok(
                        $('.ep-cke-full:visible').length === 1,
                        'Fullscreen mode is visible'
                    );
                    var buttons = $('#ep-cke-full-toolbar .cke_button .cke_button_label, #ep-cke-full-toolbar .cke_combo .cke_combo_label');

                    var labels = $.map(buttons, function (node) {
                        return node.innerHTML;
                    });

                    var expected_buttons = ["Rückgängig", "Wiederherstellen", "Suchen", "Format", "Schriftart", "Größe", "Formatierungen entfernen", "Fett", "Kursiv", "Unterstrichen", "Durchgestrichen", "Textfarbe", "Hintergrundfarbe", "Nummerierte Liste", "Liste", "Einzug verringern", "Einzug erhöhen", "Linksbündig", "Zentriert", "Rechtsbündig", "Blocksatz", "Zeilenabstand", "Bild", "App", "Link einfügen/editieren", "Link entfernen", "Tabelle", "Anker einfügen/editieren", "Horizontale Linie einfügen", "Trennzeichen einfügen", "Sonderzeichen einfügen/editieren", "Blöcke anzeigen", "Quellcode", "Verkleinern"];
                    QUnit.deepEqual(labels, expected_buttons, 'Buttons in Fullscreen in the right order');
                },
                function (data) {
                    QUnit.ok(
                        $('.ep-cke-full:visible').length === 0,
                        'Fullscreen mode is not visible'
                    );
                },
                function () {
                    ep('#uieditor_ckeditortestinstance').uiEditor('destroy');
                    setTimeout(QUnit.start, 200);
                });
        }, 3000);
    });

    QUnit.asyncTest("email fs toolbar test with placholders", function () {
        QUnit.fixtureVar.init();
        ep('#uieditor_ckeditortestinstance').uiEditor({
            'name' : 'ckeditortestinstance',
            'toolbar': 'FullscreenEmailPreview',
            pluginOptions : {
                epalmostfullscreen: {toolbar : "FullscreenEmailPreview"},
                epplaceholder : {
                    placeholder: [{"description": "Order number", "code": "#Alias"}, {"description": "Order date", "code": "#CreationDate[date]"}]
                },
                showPreview: true
            }
        });
        setTimeout(function () {
            cke_test(CKEDITOR.instances.uieditor_ckeditortestinstance,
                '.cke_button__epalmostfullscreen_icon', '.cke_button__epalmostfullscreenclose_icon',
                function () {
                    QUnit.ok(
                        $('.ep-cke-full:visible').length === 1,
                        'Fullscreen mode is visible'
                    );
                    var buttons = $('#ep-cke-full-toolbar .cke_button .cke_button_label, #ep-cke-full-toolbar .cke_combo .cke_combo_label');

                    var labels = $.map(buttons, function (node) {
                        return node.innerHTML;
                    });

                    var expected_buttons = ["Rückgängig", "Wiederherstellen", "Suchen", "Format", "Schriftart", "Größe", "Formatierungen entfernen", "Fett", "Kursiv", "Unterstrichen", "Durchgestrichen", "Textfarbe", "Hintergrundfarbe", "Nummerierte Liste", "Liste", "Einzug verringern", "Einzug erhöhen", "Linksbündig", "Zentriert", "Rechtsbündig", "Blocksatz", "Zeilenabstand", "Bild", "App", "Link einfügen/editieren", "Link entfernen", "Tabelle", "Anker einfügen/editieren", "Horizontale Linie einfügen", "Sonderzeichen einfügen/editieren", "Blöcke anzeigen", "Quellcode", "Platzhalter", "Verkleinern"];
                    QUnit.deepEqual(
                        labels,
                        expected_buttons,
                        'Buttons in Fullscreen in the right order'
                    );
                },
                function (data) {
                    QUnit.ok(
                        $('.ep-cke-full:visible').length === 0,
                        'Fullscreen mode is not visible'
                    );
                },
                function () {
                    ep('#uieditor_ckeditortestinstance').uiEditor('destroy');
                    setTimeout(QUnit.start, 200);
                });
        }, 3000);
    });
});