define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!vendor/focus-visible-test',
    'jquery',
    'vendor/focus-visible',

    '$ready!'
], function (QUnit, fixture, $, focusVisible) {

    QUnit.module('vendor.focus-visible', {
        teardown: fixture.destroy
    });

    QUnit.fixtureVar = fixture;

    QUnit.test("test focus-visible polyfill", function () {
        // clear HTML fragments and initiate fixture
        $('#qunit-fixture').nextAll().remove();
        QUnit.fixtureVar.init();

        // execute polyfill script
        focusVisible();

        // input fields for different formats
        var inputs = [
            $input_text = $('#input-text-id'),
            $input_search = $('#input-search-id'),
            $input_url = $('#input-url-id'),
            $input_tel = $('#input-tel-id'),
            $input_email = $('#input-email-id'),
            $input_password = $('#input-password-id'),
            $input_number = $('#input-number-id'),
            $input_date = $('#input-date-id'),
            $input_month = $('#input-month-id'),
            $input_week = $('#input-week-id'),
            $input_time = $('#input-time-id'),
            $input_datetime = $('#input-datetime-id'),
          ],
          // input elements with special interfaces
          $radio = $('#radio-id'),
          $checkbox = $('#checkbox-id'),
          $submit = $('#submit-id'),
          $reset = $('#reset-id'),
          // other form elements
          $textarea = $('#textarea-id'),
          $select = $('#select-id'),
          $button = $('#button-id'),
          // button element
          $buttonElem = $('#button-elem-id'),
          // elements using href
          $link = $('#link-id'),
          $area = $('#area-id'),
          // special elements: iframe
          $iframe = $('#iframe-id');

        /* Testing: input fields for different formats */
        inputs.forEach(function(input) {
          // test inputs
          input.trigger('focus');
          QUnit.ok(
            input.hasClass("focus-visible"),
            "input (type '" + input.attr('type') + "') has class 'focus-visible'"
          );
          QUnit.ok(
            input.data("focusVisibleAdded") !== undefined,
            "input (type '" + input.attr('type') + "') has data attribute 'data-focus-visible-added'"
          );
        });

        /* Testing: input elements with special interfaces */
        // test radio
        $radio.trigger('focus');
        QUnit.ok(
          $radio.hasClass("focus-visible"),
          "radio element has class 'focus-visible'"
        );
        QUnit.ok(
          $radio.data("focusVisibleAdded") !== undefined,
          "radio element has data attribute 'data-focus-visible-added'"
        );
        // test checkbox
        $checkbox.trigger('focus');
        QUnit.ok(
          $checkbox.hasClass("focus-visible"),
          "checkbox element has class 'focus-visible'"
        );
        QUnit.ok(
          $checkbox.data("focusVisibleAdded") !== undefined,
          "checkbox element has data attribute 'data-focus-visible-added'"
        );
        // test submit
        $submit.trigger('focus');
        QUnit.ok(
          $submit.hasClass("focus-visible"),
          "submit element has class 'focus-visible'"
        );
        QUnit.ok(
          $submit.data("focusVisibleAdded") !== undefined,
          "submit element has data attribute 'data-focus-visible-added'"
        );
        // test reset
        $reset.trigger('focus');
        QUnit.ok(
          $reset.hasClass("focus-visible"),
          "reset element has class 'focus-visible'"
        );
        QUnit.ok(
          $reset.data("focusVisibleAdded") !== undefined,
          "reset element has data attribute 'data-focus-visible-added'"
        );
        // test button
        $button.trigger('focus');
        QUnit.ok(
          $button.hasClass("focus-visible"),
          "input-button element has class 'focus-visible'"
        );
        QUnit.ok(
          $button.data("focusVisibleAdded") !== undefined,
          "input-button element has data attribute 'data-focus-visible-added'"
        );

        /* Testing: other form elements */
        // test textarea
        $textarea.trigger('focus');
        QUnit.ok(
          $textarea.hasClass("focus-visible"),
          "textarea element has class 'focus-visible'"
        );
        QUnit.ok(
          $textarea.data("focusVisibleAdded") !== undefined,
          "textarea element has data attribute 'data-focus-visible-added'"
        );
        // test select
        $select.trigger('focus');
        QUnit.ok(
          $select.hasClass("focus-visible"),
          "select element has class 'focus-visible'"
        );
        QUnit.ok(
          $select.data("focusVisibleAdded") !== undefined,
          "select element has data attribute 'data-focus-visible-added'"
        );

        /* Testing: button element */
        // test button element
        $buttonElem.trigger('focus');
        QUnit.ok(
          $buttonElem.hasClass("focus-visible"),
          "button element has class 'focus-visible'"
        );
        QUnit.ok(
          $buttonElem.data("focusVisibleAdded") !== undefined,
          "button element has data attribute 'data-focus-visible-added'"
        );

        /* Testing: elements using href */
        // test link
        $link.trigger('focus');
        QUnit.ok(
          $link.hasClass("focus-visible"),
          "link element has class 'focus-visible'"
        );
        QUnit.ok(
          $link.data("focusVisibleAdded") !== undefined,
          "link element has data attribute 'data-focus-visible-added'"
        );
        // test area
        $area.trigger('focus');
        QUnit.ok(
          $area.hasClass("focus-visible"),
          "area element has class 'focus-visible'"
        );
        QUnit.ok(
          $area.data("focusVisibleAdded") !== undefined,
          "area element has data attribute 'data-focus-visible-added'"
        );

        /* Testing: special elements */
        // test iframe
        $iframe.trigger('focus');
        QUnit.ok(
          $iframe.hasClass("focus-visible"),
          "iframe element has class 'focus-visible'"
        );
        QUnit.ok(
          $iframe.data("focusVisibleAdded") !== undefined,
          "iframe element has data attribute 'data-focus-visible-added'"
        );
    });
});